using Microsoft.Xaml.Interactivity;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Composition;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Hosting;
using Windows.UI.Xaml.Input;

namespace Test
{
    public class AnimationFlipViewBehavior : DependencyObject, IBehavior
    {
        FlipView flipView;
        ScrollViewer scrollViewer;
        Compositor compositor;
        CompositionPropertySet scrollPropSet;
        ExpressionAnimation CenterPointAnimation;
        ExpressionAnimation ScaleAnimation;

        public DependencyObject AssociatedObject { get; private set; }

        public void Attach(DependencyObject associatedObject)
        {
            AssociatedObject = associatedObject;
            if (associatedObject is FlipView flip) flipView = flip;
            else throw new ArgumentException("对象不是FlipView");
            scrollViewer = Helper.FindVisualChild<ScrollViewer>(flipView, "ScrollingHost");
            if (scrollViewer == null)
            {
                flipView.Loaded += FlipView_Loaded;
            }
            else InitCompositionResources(scrollViewer);
            flipView.SelectionChanged += FlipView_SelectionChanged;
        }

        public void Detach()
        {
            if (flipView != null)
            {
                flipView.Loaded -= FlipView_Loaded;
                flipView.SelectionChanged -= FlipView_SelectionChanged;
            }
        }

        void InitCompositionResources(ScrollViewer scroll)
        {
            if (compositor == null) compositor = ElementCompositionPreview.GetElementVisual(flipView).Compositor;
            if (scroll == null) return;

            scrollPropSet = ElementCompositionPreview.GetScrollViewerManipulationPropertySet(scrollViewer);
            if (CenterPointAnimation == null)
            {
                CenterPointAnimation = compositor.CreateExpressionAnimation("Vector3(visual.Size.X/2,visual.Size.Y/2,0)");
            }
            if (ScaleAnimation == null)
            {
                ScaleAnimation = compositor.CreateExpressionAnimation("Clamp(1- (visual.Offset.X + scroll.Translation.X) / visual.Size.X * 0.4, 0f, 1f)");
                //ScaleXAnimation = compositor.CreateExpressionAnimation("scroll.Translation.X % visual.Size.X < 0 ? ( -scroll.Translation.X % visual.Size.X) / visual.Size.X > 0.5 ? ( -scroll.Translation.X % visual.Size.X) / visual.Size.X : 1- ( -scroll.Translation.X % visual.Size.X) / visual.Size.X : 1");
                ScaleAnimation.SetReferenceParameter("scroll", scrollPropSet);
            }
        }

        void InitAnimation()
        {
            if (compositor != null)
            {
                for (int i = 0; i < flipView.Items.Count; i++)
                {
                    var item = flipView.ContainerFromIndex(i);
                    if (item is UIElement ele)
                    {
                        var visual = ElementCompositionPreview.GetElementVisual(ele);
                        CenterPointAnimation.SetReferenceParameter("visual", visual);
                        visual.StartAnimation("CenterPoint", CenterPointAnimation);
                        visual.StopAnimation("Scale.X");

                        ScaleAnimation.SetReferenceParameter("visual", visual);
                        visual.StartAnimation("Scale.X", ScaleAnimation);
                        visual.StartAnimation("Scale.Y", ScaleAnimation);
                    }
                }
            }
        }

        private void FlipView_Loaded(object sender, RoutedEventArgs e)
        {
            flipView.Loaded -= FlipView_Loaded;
            var scroll = Helper.FindVisualChild<ScrollViewer>(flipView, "ScrollingHost");
            if (scroll == null) throw new ArgumentNullException("ScrollViewer为空");
            else scrollViewer = scroll;

            InitCompositionResources(scrollViewer);
            InitAnimation();
        }

        private void FlipView_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            InitAnimation();
        }

    }


        public static class Helper
    {
        public static T FindVisualChild<T>(DependencyObject obj, int Index = 0) where T : DependencyObject
        {
            if (Index == -1) return null;
            int count = VisualTreeHelper.GetChildrenCount(obj);
            int findedcount = 0;
            for (int i = 0; i < count; i++)
            {
                DependencyObject child = Windows.UI.Xaml.Media.VisualTreeHelper.GetChild(obj, i);
                if (child != null && child is T)
                {
                    if (findedcount == Index)
                        return (T)child;
                    else
                    {
                        findedcount++;
                    }
                }
                else
                {
                    T childOfChild = FindVisualChild<T>(child, findedcount);
                    if (childOfChild != null)
                        return childOfChild;
                }
            }
            return null;
        }
        public static T FindVisualChild<T>(DependencyObject obj, string name) where T : DependencyObject
        {
            int count = VisualTreeHelper.GetChildrenCount(obj);
            int findedcount = 0;
            for (int i = 0; i < count; i++)
            {
                DependencyObject child = Windows.UI.Xaml.Media.VisualTreeHelper.GetChild(obj, i);
                if (child != null && child is T)
                {
                    if ((child as FrameworkElement).Name == name)
                        return (T)child;
                    else
                    {
                        findedcount++;
                    }
                }
                else
                {
                    T childOfChild = FindVisualChild<T>(child, findedcount);
                    if (childOfChild != null)
                        return childOfChild;
                }
            }
            return null;
        }
    }
}
